/**
 * @class CommentData
 * @mixes fraudCommons/util/BaseDataMixin
 * @description Helper module for executing servlet requests and invoking callbacks to handle response data.
 */
define([
    'dojo/Deferred',
    'dojox/lang/functional',
    'dojo/_base/lang',
    'dojo/request',
    'fraudCommons/util/BaseDataMixin'
], function(Deferred, functional, lang, request, BaseDataMixin) {

    var JSON_COMMENTS = "src/fraudCommons/widgets/CommentManager/json/getCommentManager.json";
    var data = lang.mixin({}, new BaseDataMixin());

    /**
     * @description - Pre-production testing helper function to allow JSON data to be used instead of handling AJAX
     *                calls and response data.
     * @param {boolean} state - True to enable debug mode
     */
    data.setDebug = function(state) {
        this.debug = state;
    };

    /**
     * @description Updates attachments for an object.
     * @param {Object|number} json - Data for the object to modify. If the comment already exists, this value should be
     *                               the ID of the comment, otherwise it should be an object with all relevant
     *                               data necessary to create a new comment.
     * @param {boolean} newComment - True if creating a new comment.
     * @param {function} [callback] - Function to execute once the response is returned. Response data is passed in as
     *                                an argument to the callback.
     */
    data.hasAttachments = function(json, newComment, callback) {
        var promise = new Deferred();
        if(!this.debug) {
            var url = 'comments';
            var q = {
                action: 'hasAttachments'
            };
            var data;
            if(newComment) {
                q.isNewComment = true;
                functional.forIn(json, function(val, key) {
                    q[key] = val;
                });
            } else {
                q.id = json; //actually just a long
            }
            request.post(url, {
                query: q
            }).then(function(data) {
                if(callback) {
                    callback(data);
                }
                promise.resolve();
            });
        }
        return promise;
    };

    /**
     * @description Creates a new comment.
     * @param {Object} json - Data for the object to modify.
     * @param {function} [callback] - Function to execute once the response is returned. Response data is passed in as
     *                                an argument to the callback.
     */
    data.save = function(json, callback) {
        var promise = new Deferred();
        if(!this.debug) {
            var url = 'comments';
            var d = {
                action: 'new'
            };
            functional.forIn(json, function(val, key) {
                d[key] = val;
            });
            request.post(url, {
                handleAs: "json",
                data: d
            }).then(function(data) {
                if(callback) {
                    callback(data);
                }
                promise.resolve();
            });
        }
        return promise;
    };

    /**
     * @description Updates an existing comment.
     * @param {Object} json - Data for the object to modify.
     * @param {function} [callback] - Function to execute once the response is returned. Response data is passed in as
     *                                an argument to the callback.
     */
    data.update = function(json, callback) {
        var promise = new Deferred();
        if(!this.debug) {
            var url = 'comments';
            var d = {
                action: 'update'
            };
            functional.forIn(json, function(val, key) {
                d[key] = val;
            });
            request.post(url, {
                handleAs: "json",
                data: d
            }).then(function(data) {
                if(callback) {
                    callback(data);
                }
                promise.resolve();
            });
        }
        return promise;
    };

    /**
     * @description Request all available comments from the midtier in JSON form.
     * @param {number|string} objectId - The ID of the object for which to retrieve comments.
     * @param {number|string} objectType - The type of the object for which to retrieve comments.
     * @param {string} objectName - The name of the object for which to retrieve comments.
     * @param {function} [callback] - Function to execute once the response is returned. Response data is passed in as
     *                                an argument to the callback.
     */
    data.retrieve = function(objectId, objectType, objectName, successCallback, errorCallback) {
        var promise = new Deferred();
        var url = 'comments';
        if(this.debug) {
            url = JSON_COMMENTS;
        }
        request.post(url, {
            handleAs: "json",
            query: {
                action: 'retrieve',
                objectId: objectId,
                objectType: objectType,
                objectName: objectName,
                noCacheFlag: new Date()
            }
        }).then(function(data) {
            if(successCallback) {
                successCallback(data);
            }
            promise.resolve();
        }, function(err) { //handle error
            if(errorCallback) {
                errorCallback(err);
            }
        });
        return promise;
    };

    /**
     * @description Removes attachments from the comment with the given ID.
     * @param {array} attachments - The attachments to remove from the comment.
     * @param {number} commentId - The ID of the comment from which attachments will be deleted.
     * @param {function} [callback] - Function to execute once the response is returned. Response data is passed in as
     *                                an argument to the callback.
     */
    data.deleteAttachments = function(attachments, commentId, callback) {
        var promise = new Deferred();
        var url = 'comments';
        if(this.debug) {
            url = JSON_COMMENTS;
        }
        request.post(url, {
            handleAs: "json",
            query: {
                action: 'deleteAttachments',
                attachments: JSON.stringify(attachments),
                commentId: commentId
            }
        }).then(function(data) {
            if(callback) {
                callback(data);
            }
            promise.resolve();
        });
        return promise;
    };

    /**
     * @description Deletes a comment with the given ID.
     * @param {number} id - The ID of the comment to delete.
     * @param {function} [callback] - Function to execute once the response is returned. Response data is passed in as
     *                                an argument to the callback.
     */
    data.deleteComment = function(id, callback) {
        var promise = new Deferred();
        var url = 'comments';
        if(this.debug) {
            url = JSON_COMMENTS;
        }
        request.post(url, {
            handleAs: "json",
            query: {
                action: 'deleteComment',
                id: id
            }
        }).then(function(data) {
            if(callback) {
                callback(data);
            }
            promise.resolve();
        });
        return promise;
    };

    return data;
});
